<?php
/**
 * @file
 * Menu callbacks.
 */

/**
 * Settings page callback.
 *
 * @see markdownizer_menu()
 */
function markdownizer_settings_form(array $form, array &$form_sate) {
  $form = array_filter(_markdownizer_get_settings(FALSE));

  return empty($form) ? $form : system_settings_form($form);
}

/**
 * Get saved settings or configuration form.
 *
 * @param bool $get_settings
 *   When FALSE, then configuration form will be returned.
 *
 * @return array
 *   An array with settings or configuration form.
 */
function _markdownizer_get_settings($get_settings = TRUE) {
  $settings = [];
  $form = [];

  foreach (markdownizer_libraries_info() as $library_name => $definition) {
    $form[$library_name] = markdownizer_libraries('detect', $library_name, function (array $library, array $options) use ($definition, $library_name, &$settings) {
      if (empty($definition['#form']) || !empty($library['error'])) {
        return [];
      }

      foreach ($definition['#form'] as $field_name => &$field) {
        // Update the #default_value from settings, if it exist.
        if (isset($options[$field_name])) {
          $field['#default_value'] = $options[$field_name];
        }

        // Set the default value to $settings array.
        $settings[$library_name][$field_name] = $field['#default_value'];
        // Execute callback function for processing an element if it exist.
        $callback = sprintf('_%s_%s_%s_form_element_callback', MARKDOWNIZER_MODULE_NAME, $library_name, $field_name);

        if (function_exists($callback)) {
          $callback($field, $library, $settings);
        }
      }

      return array_merge($definition['#form'], [
        '#tree' => TRUE,
        '#type' => 'fieldset',
        '#title' => $library['name'],
      ]);
    });
  }

  return $get_settings ? $settings : $form;
}

/**
 * Form element callback.
 *
 * @param array $field
 *   Form field.
 * @param array $library
 *   Library definition, returned by {@link libraries_detect()} function.
 * @param array $settings
 *   Settings of library, configured in form in UI.
 */
function _markdownizer_highlightjs_style_form_element_callback(array &$field, array $library, array $settings) {
  $styles_path = $library['library path'] . '/build/styles';

  if (file_exists($styles_path)) {
    $cache = cache_get($styles_path) ?: new \stdClass();

    if (empty($cache->data)) {
      $cache->data = [];

      /* @var \SplFileInfo $info */
      foreach (new \FilesystemIterator($styles_path) as $path => $info) {
        // Remove ".min.css", ".min" or ".css" from file name and
        // split style name and color.
        $names = explode('.', str_replace(['.min', '.css'], '', $info->getFilename()));
        $scheme = '';

        // If style name contain color scheme then format it.
        if (count($names) > 1) {
          list($name, $scheme) = $names;
          $scheme = ' [' . ucfirst($scheme) . ']';
        }
        else {
          $name = reset($names);
        }

        // @example
        // "style-name.dark.css" will be transformed to "Style name [Dark]".
        $cache->data[$path] = ucfirst(str_replace(['-', '_'], ' ', $name)) . $scheme;
      }

      cache_set($styles_path, $cache->data);
    }

    $field = array_merge($field, [
      '#options' => $cache->data,
      '#description' => t('To see how styles look, visit the !lib home page.', [
        '!lib' => l($library['name'], $library['vendor url']),
      ]),
    ]);
  }
}
